/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.gain;

import java.io.IOException;

import org.audiochain.io.AbstractAudioDataReaderChainLink;
import org.audiochain.io.AudioDataReader;

/**
 * This is a very simple example for an AudioDataReader. It manipulates the
 * audio data while it multiplies each sample by an adjustable factor.
 */
public class GainAudioDataReader extends AbstractAudioDataReaderChainLink {
	private float gainFactor;

	public GainAudioDataReader(AudioDataReader source, float gain) {
		super(source);
		setGain(gain);
	}

	@Override
	public int read(int[] data) throws IOException {
		/*
		 * Read the audio data from the implicitly visible source
		 * AudioDataReader. The data array is shared over the whole reader chain
		 * this means it is shared by all readers that are calling this read
		 * method. The source may be changed from external.
		 */
		int len = source.read(data);
		/*
		 * Modify the read data, here we simply multiply the samples with a
		 * factor. This algorithm must be coded very efficiently. As an example,
		 * this saves cpu resources if gain factor is exactly 1.
		 */
		if (gainFactor > 1) {
			for (int i = 0; i < len; i++) {
				data[i] *= gainFactor;
			}
		}
		// return the length of the read samples
		return len;
	}

	public void setGain(float gain) {
		/*
		 * Convert gain in dB (Decibel) to a factor to save cpu resources while
		 * reading and processing the audio data.
		 */
		float exp = gain / 20F;
		gainFactor = (float) Math.pow(10D, exp);
	}
}
