/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.gain;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.FrameObserver;
import org.audiochain.model.AbstractChainableAudioDevice;
import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceProperty;
import org.audiochain.model.UserInterfaceContext;
import org.audiochain.ui.gui.GenericUserInterfaceContext;

/**
 * This is a very simple audio device that has one property named 'Gain'.
 * 
 * To integrate this audio device into AudioChain, run the JVM with the following parameter:
 * 
 * <pre>-DadditionalAudioDeviceClasses=org.audiochain.devices.gain.GainAudioDevice</pre>
 * 
 */
public class GainAudioDevice extends AbstractChainableAudioDevice {
	private static final long serialVersionUID = 1L;

	/*
	 * This property is manipulatable by the user. It must have a name.
	 * Optionally it can have a min and max value which influences the UI
	 * validation and visible components or labels. The unit is a plain visible
	 * artefact and has no technical consequences. The symbol is displayed in
	 * the command line interface (CLI) and means a unicode character.
	 * 
	 * The audio device becomes stored (persisted) so the properties must not be
	 * transient.
	 */
	@AudioDeviceProperty(name = "Gain", max = 30, min = 0, unit = "dB", symbol = 9651)
	private float gain = 0F;
	/* The reader must be transient to not become saved. */
	private transient GainAudioDataReader reader;

	@Override
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		/*
		 * Create a new reader and remember it in a transient (!!!) instance
		 * variable for manipulating the reader while it is in use (playback).
		 */
		reader = new GainAudioDataReader(source, gain);
		return reader;
	}

	/**
	 * This is the name, displayed to the user on the user interface (GUI or
	 * CLI).
	 */
	@Override
	public String getName() {
		return "Gain";
	}

	@Override
	protected UserInterfaceContext<? extends AudioDevice> createUserInterfaceContext() {
		/*
		 * The simplest way to provide a GUI and a CLI. If the UI has to be more
		 * complex, you can implement the UserInterfaceContext interface
		 * directly.
		 */
		return new GenericUserInterfaceContext<AudioDevice>();
	}

	/**
	 * Because the gain variable is annotated with the @AudioDeviceProperty,
	 * this setter is called 'by magic' when the user manipulates the property
	 * through the UI or through the sync process.
	 * 
	 * If the reader is available (this means, it is playing back at this
	 * moment), the property change should be promoted to the reader
	 * immediately, so the user can hear the changes directly.
	 * 
	 * @param gain
	 */
	public void setGain(float gain) {
		this.gain = gain;
		if (reader != null) {
			reader.setGain(gain);
		}
	}
}
