/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.vst;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.audiochain.io.AudioDataReader;
import org.audiochain.io.AudioDataReaderChainLink;
import org.audiochain.io.DelegatingAudioDataReader;
import org.audiochain.io.FrameObserver;
import org.audiochain.model.AbstractChainableAudioDevice;
import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceValue;
import org.audiochain.model.AudioDeviceValueListener;
import org.audiochain.model.AudioProject;
import org.audiochain.model.Copyable;
import org.audiochain.model.FloatAudioDeviceValue;
import org.audiochain.model.Synchronizable;
import org.audiochain.model.UserInterfaceContext;
import org.audiochain.ui.gui.GenericUserInterfaceContext;

import com.synthbot.audioplugin.vst.vst2.JVstHost2;

public class ChainableVstAudioDevice extends AbstractChainableAudioDevice implements AudioDeviceValueListener, Copyable<ChainableVstAudioDevice> {
	private static final long serialVersionUID = 1L;
	private final String vstPluginIdentifier;
	private String vstPluginName;
	private Map<String, Float> values = new LinkedHashMap<String, Float>();
	private transient JVstHost2 vstHost;
	private transient List<FloatAudioDeviceValue> parameterIndexLookup;

	@Deprecated
	ChainableVstAudioDevice() {
		// only for deserialization
		vstPluginIdentifier = null;
	}

	public ChainableVstAudioDevice(String vstPluginIdentifier) {
		this.vstPluginIdentifier = vstPluginIdentifier;
	}

	@Override
	public void initialize(AudioProject audioProject) {
		super.initialize(audioProject);
		vstHost = VstPluginLibrary.getInstance().getVstPlugin(vstPluginIdentifier, audioProject);
		if (vstHost == null) {
			// the plugin may be removed, so we have to deal with this situation
			// gracefully
			setEnabled(false);
		} else {
			vstPluginName = vstHost.getEffectName();
		}
	}

	@Override
	public AudioDataReaderChainLink createAudioDataReaderChainLink(AudioDataReader source, float frameRate, int sampleSizeInBits, FrameObserver frameObserver) {
		if (vstHost == null) {
			return new DelegatingAudioDataReader(source);
		}
		return new VstAudioDataReaderChainLink(source, vstHost);
	}

	@Override
	protected void createAudioDeviceValues(Collection<AudioDeviceValue> audioDeviceValues) {
		if (vstHost == null) {
			return;
		}
		int numParameters = vstHost.numParameters();
		parameterIndexLookup = new ArrayList<FloatAudioDeviceValue>();
		for (int i = 0; i < numParameters; i++) {
			String parameterName = vstHost.getParameterName(i);
			parameterName = parameterName.trim();
			Float parameterValue = values.get(parameterName);
			if (parameterValue == null) {
				parameterValue = vstHost.getParameter(i);
			} else {
				vstHost.setParameter(i, parameterValue);
			}
			FloatAudioDeviceValue val = new FloatAudioDeviceValue(this, parameterName, parameterValue);
			String parameterLabel = vstHost.getParameterLabel(i);
			String parameterDisplay = vstHost.getParameterDisplay(i);
			val.setUnit(parameterLabel + " " + parameterDisplay);
			val.setMinValue(0f);
			val.setMaxValue(1f);
			audioDeviceValues.add(val);
			val.addAudioDeviceValueListener(this);
			parameterIndexLookup.add(val);
		}
	}

	@Override
	public void audioDeviceValueChanged(AudioDevice audioDevice, AudioDeviceValue audioDeviceValue, String oldValue, String newValue) {
		String parameterName = audioDeviceValue.getName();
		if (newValue == null) {
			values.put(parameterName, null);
		} else {
			Float floatValue = Float.valueOf(newValue);
			values.put(parameterName, floatValue);
			int parameterIndex = parameterIndexLookup.indexOf(audioDeviceValue);
			vstHost.setParameter(parameterIndex, floatValue);
			String parameterLabel = vstHost.getParameterLabel(parameterIndex);
			String parameterDisplay = vstHost.getParameterDisplay(parameterIndex);
			audioDeviceValue.setUnit(parameterLabel.trim() + " " + parameterDisplay.trim());
		}
		firePerformanceRelevantAudioDevicePropertyChanged();
		firePersistableAudioDevicePropertyChanged();
	}

	@Override
	public String getName() {
		return vstPluginName + (vstHost == null ? " (N/A)" : "");
	}

	@Override
	protected UserInterfaceContext<? extends AudioDevice> createUserInterfaceContext() {
		return new GenericUserInterfaceContext<AudioDevice>();
	}

	@Override
	public ChainableVstAudioDevice copy() {
		ChainableVstAudioDevice ret = new ChainableVstAudioDevice(vstPluginIdentifier);
		ret.vstPluginName = vstPluginName;
		ret.values.putAll(values);
		return ret;
	}

	@Override
	public void synchronizeBy(Synchronizable source) {
		super.synchronizeBy(source);
		ChainableVstAudioDevice other = (ChainableVstAudioDevice) source;
		if (other.vstHost == null) { // remote objects do not have a vstHost, so
										// we have to sync by hand
			for (Entry<String, Float> otherValuePair : other.values.entrySet()) {
				String valueName = otherValuePair.getKey();
				long sourceTime = other.getAudioDeviceValueLastChangeTime(valueName);
				long targetTime = getAudioDeviceValueLastChangeTime(valueName);
				if (greaterOrBothZero(sourceTime, targetTime) && other.vstPluginIdentifier.equals(vstPluginIdentifier)) {
					Collection<AudioDeviceValue> audioDeviceValues = getAudioDeviceValues();
					for (AudioDeviceValue audioDeviceValue : audioDeviceValues) {
						if (audioDeviceValue.getName().equals(valueName) && audioDeviceValue instanceof FloatAudioDeviceValue) {
							FloatAudioDeviceValue floatValue = (FloatAudioDeviceValue) audioDeviceValue;
							Float value = otherValuePair.getValue();
							floatValue.setValue(value == null ? 0 : value);
							setAudioDeviceValueLastChangeTime(valueName, sourceTime);
						}
					}
				}
			}
		}
	}
}
