/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.vst;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.audiochain.model.AudioDeviceFactory.AudioDeviceIdentifier;
import org.audiochain.model.AudioMixer;
import org.audiochain.model.AudioProject;

import com.synthbot.audioplugin.vst.vst2.JVstHost2;

public class VstPluginLibrary {
	private static final String VST_PLUGIN_DIR = "vst.plugin.dir";
	private static VstPluginLibrary instance;
	private final Map<String, File> pluginFiles = new HashMap<String, File>();
	private final Collection<AudioDeviceIdentifier> vstPluginIdentifiers = new LinkedHashSet<AudioDeviceIdentifier>();

	public static VstPluginLibrary getInstance() {
		if (instance == null) {
			instance = new VstPluginLibrary();
			instance.loadLibraries();
		}
		return instance;
	}

	private void loadLibraries() {
		String pluginDirName = System.getProperty(VST_PLUGIN_DIR);
		if (pluginDirName == null) {
			throw new IllegalStateException("System property '" + VST_PLUGIN_DIR + "' is undefined.");
		}
		File dir = new File(pluginDirName);
		if (!dir.exists()) {
			throw new IllegalStateException("System property '" + VST_PLUGIN_DIR + "' points to a non existent directory: '" + dir.getAbsolutePath() + "'");
		}
		File[] listFiles = dir.listFiles();
		int bufferSize = AudioMixer.getInstance().getPlaybackBufferSizeInSamples();
		String fileEnd;
		String os = System.getProperty("os.name").toLowerCase();
		if(os.indexOf("nux")>=0) {
			fileEnd = ".so";
		} else if(os.indexOf("mac")>=0) {
			fileEnd = ".vst";
		} else if(os.indexOf("win")>=0) {
			fileEnd = ".dll";
		} else {
			fileEnd = "";
		}
		for (File file : listFiles) {
			if (file.getName().toLowerCase().trim().endsWith(fileEnd)) {
				try {
					JVstHost2 vstHost = JVstHost2.newInstance(file, 44100F, bufferSize);
					String uniqueId = vstHost.getUniqueId();
					uniqueId = uniqueId.trim();
					pluginFiles.put(uniqueId, file);
					int numInputs = vstHost.numInputs();
					String effectName = vstHost.getEffectName();
					effectName = effectName.trim();
					vstPluginIdentifiers.add(new AudioDeviceIdentifier(uniqueId, effectName, numInputs == 0));
					vstHost.turnOffAndUnloadPlugin();
				} catch (FileNotFoundException e) {
					throw new IllegalStateException(e);
				} catch (Throwable e) {
					System.err.println("Unable to load file '" + file.getAbsolutePath() + "'.");
					e.printStackTrace();
				}
			}
		}
	}

	public JVstHost2 getVstPlugin(String vstPluginId, AudioProject audioProject) {
		File file = pluginFiles.get(vstPluginId);
		if (file == null) {
			return null;
		}
		int frameRate = audioProject.getFrameRate();
		int bufferSize = AudioMixer.getInstance().getPlaybackBufferSizeInSamples();
		try {
			JVstHost2 vstHost = JVstHost2.newInstance(file, frameRate, bufferSize);
			return vstHost;
		} catch (Throwable e) {
			throw new IllegalStateException(e);
		}
	}

	public Collection<AudioDeviceIdentifier> getAudioDeviceIdentifiers() {
		return Collections.unmodifiableCollection(vstPluginIdentifiers);
	}
}
